@@ -1,4 +1,4 @@
-# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 # copyright 2003-2010 Sylvain Thenault, all rights reserved.
 # contact mailto:thenault@gmail.com
@@ -22,10 +22,11 @@
 
 import sys
 
-from logilab.astng import NoDefault
-from logilab.astng.bases import NodeNG, BaseClass, Instance, copy_context, \
-                                _infer_stmts, YES
-from logilab.astng.mixins import StmtMixIn, BlockRangeMixIn, AssignTypeMixin, \
+from logilab.astng import BUILTINS_MODULE
+from logilab.astng.exceptions import NoDefault
+from logilab.astng.bases import (NodeNG, Statement, Instance, InferenceContext,
+                                 _infer_stmts, YES)
+from logilab.astng.mixins import BlockRangeMixIn, AssignTypeMixin, \
                                  ParentAssignTypeMixin, FromImportMixIn
 
 
@@ -98,7 +99,7 @@ def are_exclusive(stmt1, stmt2, exceptions=None):
     return False
 
 
-class LookupMixIn(BaseClass):
+class LookupMixIn(object):
     """Mixin looking up a name in the right scope
     """
 
@@ -115,15 +116,14 @@ class LookupMixIn(BaseClass):
         """
         return self.scope().scope_lookup(self, name)
 
-    def ilookup(self, name, context=None):
+    def ilookup(self, name):
         """infered lookup
 
         return an iterator on infered values of the statements returned by
         the lookup method
         """
         frame, stmts = self.lookup(name)
-        context = copy_context(context)
-        context.lookupname = name
+        context = InferenceContext()
         return _infer_stmts(stmts, context, frame)
 
     def _filter_stmts(self, stmts, frame, offset):
@@ -333,19 +333,19 @@ class AssAttr(NodeNG, ParentAssignTypeMixin):
     _astng_fields = ('expr',)
     expr = None
 
-class Assert(StmtMixIn, NodeNG):
+class Assert(Statement):
     """class representing an Assert node"""
     _astng_fields = ('test', 'fail',)
     test = None
     fail = None
 
-class Assign(StmtMixIn, NodeNG, AssignTypeMixin):
+class Assign(Statement, AssignTypeMixin):
     """class representing an Assign node"""
     _astng_fields = ('targets', 'value',)
     targets = None
     value = None
 
-class AugAssign(StmtMixIn, NodeNG, AssignTypeMixin):
+class AugAssign(Statement, AssignTypeMixin):
     """class representing an AugAssign node"""
     _astng_fields = ('target', 'value',)
     target = None
@@ -367,7 +367,7 @@ class BoolOp(NodeNG):
     _astng_fields = ('values',)
     values = None
 
-class Break(StmtMixIn, NodeNG):
+class Break(Statement):
     """class representing a Break node"""
 
 
@@ -428,14 +428,14 @@ class Comprehension(NodeNG):
 
 
 class Const(NodeNG, Instance):
-    """represent a Str or Num node"""
+    """represent a constant node like num, str, bool, None, bytes"""
 
     def __init__(self, value=None):
         self.value = value
 
     def getitem(self, index, context=None):
         if isinstance(self.value, basestring):
-            return self.value[index]
+            return Const(self.value[index])
         raise TypeError('%r (value=%s)' % (self, self.value))
 
     def has_dynamic_getattr(self):
@@ -446,7 +446,11 @@ class Const(NodeNG, Instance):
             return self.value
         raise TypeError()
 
-class Continue(StmtMixIn, NodeNG):
+    def pytype(self):
+        return self._proxied.qname()
+
+
+class Continue(Statement):
     """class representing a Continue node"""
 
 
@@ -468,7 +472,7 @@ class DelAttr(NodeNG, ParentAssignTypeMixin):
     expr = None
 
 
-class Delete(StmtMixIn, NodeNG, AssignTypeMixin):
+class Delete(Statement, AssignTypeMixin):
     """class representing a Delete node"""
     _astng_fields = ('targets',)
     targets = None
@@ -477,10 +481,16 @@ class Delete(StmtMixIn, NodeNG, AssignTypeMixin):
 class Dict(NodeNG, Instance):
     """class representing a Dict node"""
     _astng_fields = ('items',)
-    items = None
+
+    def __init__(self, items=None):
+        if items is None:
+            self.items = []
+        else:
+            self.items = [(const_factory(k), const_factory(v))
+                          for k,v in items.iteritems()]
 
     def pytype(self):
-        return '__builtin__.dict'
+        return '%s.dict' % BUILTINS_MODULE
 
     def get_children(self):
         """get children of a Dict node"""
@@ -508,7 +518,7 @@ class Dict(NodeNG, Instance):
         raise IndexError(key)
 
 
-class Discard(StmtMixIn, NodeNG):
+class Discard(Statement):
     """class representing a Discard node"""
     _astng_fields = ('value',)
     value = None
@@ -522,7 +532,7 @@ class EmptyNode(NodeNG):
     """class representing an EmptyNode node"""
 
 
-class ExceptHandler(StmtMixIn, NodeNG, AssignTypeMixin):
+class ExceptHandler(Statement, AssignTypeMixin):
     """class representing an ExceptHandler node"""
     _astng_fields = ('type', 'name', 'body',)
     type = None
@@ -550,7 +560,7 @@ class ExceptHandler(StmtMixIn, NodeNG, AssignTypeMixin):
                 return True
 
 
-class Exec(StmtMixIn, NodeNG):
+class Exec(Statement):
     """class representing an Exec node"""
     _astng_fields = ('expr', 'globals', 'locals',)
     expr = None
@@ -563,8 +573,7 @@ class ExtSlice(NodeNG):
     _astng_fields = ('dims',)
     dims = None
 
-
-class For(BlockRangeMixIn, StmtMixIn, AssignTypeMixin, NodeNG):
+class For(BlockRangeMixIn, AssignTypeMixin, Statement):
     """class representing a For node"""
     _astng_fields = ('target', 'iter', 'body', 'orelse',)
     target = None
@@ -577,7 +586,7 @@ class For(BlockRangeMixIn, StmtMixIn, AssignTypeMixin, NodeNG):
         return self.iter.tolineno
 
 
-class From(FromImportMixIn, StmtMixIn, NodeNG):
+class From(FromImportMixIn, Statement):
     """class representing a From node"""
 
     def __init__(self,  fromname, names, level=0):
@@ -591,7 +600,7 @@ class Getattr(NodeNG):
     expr = None
 
 
-class Global(StmtMixIn, NodeNG):
+class Global(Statement):
     """class representing a Global node"""
 
     def __init__(self, names):
@@ -601,7 +610,7 @@ class Global(StmtMixIn, NodeNG):
         return name
 
 
-class If(BlockRangeMixIn, StmtMixIn, NodeNG):
+class If(BlockRangeMixIn, Statement):
     """class representing an If node"""
     _astng_fields = ('test', 'body', 'orelse')
     test = None
@@ -629,7 +638,7 @@ class IfExp(NodeNG):
     orelse = None
 
 
-class Import(FromImportMixIn, StmtMixIn, NodeNG):
+class Import(FromImportMixIn, Statement):
     """class representing an Import node"""
 
 
@@ -647,11 +656,16 @@ class Keyword(NodeNG):
 
 class List(NodeNG, Instance, ParentAssignTypeMixin):
     """class representing a List node"""
-    _astng_fields = ('elts',) 
-    elts = None
+    _astng_fields = ('elts',)
+
+    def __init__(self, elts=None):
+        if elts is None:
+            self.elts = []
+        else:
+            self.elts = [const_factory(e) for e in elts]
 
     def pytype(self):
-        return '__builtin__.list'
+        return '%s.list' % BUILTINS_MODULE
 
     def getitem(self, index, context=None):
         return self.elts[index]
@@ -660,7 +674,7 @@ class List(NodeNG, Instance, ParentAssignTypeMixin):
         return self.elts
 
 
-class Nonlocal(StmtMixIn, NodeNG):
+class Nonlocal(Statement):
     """class representing a Nonlocal node"""
 
     def __init__(self, names):
@@ -670,19 +684,18 @@ class Nonlocal(StmtMixIn, NodeNG):
         return name
 
 
-class Pass(StmtMixIn, NodeNG):
+class Pass(Statement):
     """class representing a Pass node"""
 
 
-class Print(StmtMixIn, NodeNG):
+class Print(Statement):
     """class representing a Print node"""
     _astng_fields = ('dest', 'values',)
     dest = None
     values = None
 
 
-
-class Raise(StmtMixIn, NodeNG):
+class Raise(Statement):
     """class representing a Raise node"""
     exc = None
     if sys.version_info < (3, 0):
@@ -691,8 +704,8 @@ class Raise(StmtMixIn, NodeNG):
         tback = None
     else:
         _astng_fields = ('exc', 'cause')
-        inst = None
-        tback = None
+        exc = None
+        cause = None
 
     def raises_not_implemented(self):
         if not self.exc:
@@ -702,7 +715,7 @@ class Raise(StmtMixIn, NodeNG):
                 return True
 
 
-class Return(StmtMixIn, NodeNG):
+class Return(Statement):
     """class representing a Return node"""
     _astng_fields = ('value',)
     value = None
@@ -711,10 +724,15 @@ class Return(StmtMixIn, NodeNG):
 class Set(NodeNG, Instance, ParentAssignTypeMixin):
     """class representing a Set node"""
     _astng_fields = ('elts',)
-    elts = None
+
+    def __init__(self, elts=None):
+        if elts is None:
+            self.elts = []
+        else:
+            self.elts = [const_factory(e) for e in elts]
 
     def pytype(self):
-        return '__builtin__.set' # XXX __builtin__ vs builtins
+        return '%s.set' % BUILTINS_MODULE
 
     def itered(self):
         return self.elts
@@ -740,7 +758,7 @@ class Subscript(NodeNG):
     slice = None
 
 
-class TryExcept(BlockRangeMixIn, StmtMixIn, NodeNG):
+class TryExcept(BlockRangeMixIn, Statement):
     """class representing a TryExcept node"""
     _astng_fields = ('body', 'handlers', 'orelse',)
     body = None
@@ -766,7 +784,7 @@ class TryExcept(BlockRangeMixIn, StmtMixIn, NodeNG):
         return self._elsed_block_range(lineno, self.orelse, last)
 
 
-class TryFinally(BlockRangeMixIn, StmtMixIn, NodeNG):
+class TryFinally(BlockRangeMixIn, Statement):
     """class representing a TryFinally node"""
     _astng_fields = ('body', 'finalbody',)
     body = None
@@ -788,10 +806,15 @@ class TryFinally(BlockRangeMixIn, StmtMixIn, NodeNG):
 class Tuple(NodeNG, Instance, ParentAssignTypeMixin):
     """class representing a Tuple node"""
     _astng_fields = ('elts',)
-    elts = None
+
+    def __init__(self, elts=None):
+        if elts is None:
+            self.elts = []
+        else:
+            self.elts = [const_factory(e) for e in elts]
 
     def pytype(self):
-        return '__builtin__.tuple'
+        return '%s.tuple' % BUILTINS_MODULE
 
     def getitem(self, index, context=None):
         return self.elts[index]
@@ -806,7 +829,7 @@ class UnaryOp(NodeNG):
     operand = None
 
 
-class While(BlockRangeMixIn, StmtMixIn, NodeNG):
+class While(BlockRangeMixIn, Statement):
     """class representing a While node"""
     _astng_fields = ('test', 'body', 'orelse',)
     test = None
@@ -820,7 +843,8 @@ class While(BlockRangeMixIn, StmtMixIn, NodeNG):
         """handle block line numbers range for for and while statements"""
         return self. _elsed_block_range(lineno, self.orelse)
 
-class With(BlockRangeMixIn, StmtMixIn, AssignTypeMixin, NodeNG):
+
+class With(BlockRangeMixIn, AssignTypeMixin, Statement):
     """class representing a With node"""
     _astng_fields = ('expr', 'vars', 'body')
     expr = None
@@ -845,24 +869,35 @@ CONST_CLS = {
     list: List,
     tuple: Tuple,
     dict: Dict,
+    set: Set,
+    type(None): Const,
     }
 
+def _update_const_classes():
+    """update constant classes, so the keys of CONST_CLS can be reused"""
+    klasses = (bool, int, float, complex, str)
+    if sys.version_info < (3, 0):
+        klasses += (unicode, long)
+    if sys.version_info >= (2, 6):
+        klasses += (bytes,)
+    for kls in klasses:
+        CONST_CLS[kls] = Const
+_update_const_classes()
+
 def const_factory(value):
     """return an astng node for a python value"""
+    # since const_factory is called to evaluate content of container (eg list,
+    # tuple), it may be called with some node as argument that should be left
+    # untouched
+    if isinstance(value, NodeNG):
+        return value
     try:
-        # if value is of class list, tuple, dict use specific class, not Const
-        cls = CONST_CLS[value.__class__]
-        node = cls()
-        if isinstance(node, Dict):
-            node.items = ()
-        else:
-            node.elts = ()
-    except KeyError:
-        # why was value in (None, False, True) not OK?
-        assert isinstance(value, (int, long, complex, float, basestring)) or value in (None, False, True)
-        node = Const()
-        node.value = value
-    return node
-
-
-
+        return CONST_CLS[value.__class__](value)
+    except (KeyError, AttributeError):
+        # some constants (like from gtk._gtk) don't have their class in
+        # CONST_CLS, though we can "assert isinstance(value, tuple(CONST_CLS))"
+        if isinstance(value, tuple(CONST_CLS)):
+            return Const(value)
+        node = EmptyNode()
+        node.object = value
+        return node
